#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

# This test ensures the use of the `with` statement, and thus runtime context, works as expected.

from ecflow import Defs, Suite, Task, Family, Client, Variable, Edit, debug_build
import ecflow_test_util as Test
import sys, os

if __name__ == "__main__":
    Test.print_test_start(os.path.basename(__file__))

    # sys.version_info is a tuple containing (major,minor,micro,releaselevel,serial)
    version = sys.version_info;
    if (sys.version_info < (2, 7)):
        print("This test only run with python version 2.7 or greater, but found : " + str(version))
        sys.exit(0)

    # test `add_*()` using context manager -- n.b. this provides the ability to indent

    with Defs() as defs:
        with defs.add_suite("s2") as s2:
            with s2.add_family("f1") as f1:
                with f1.add_task("t1") as t1:
                    assert len(f1) == 1, "Expected 1 task"
            s2.add_family("f2")
            s2.add_task("t1")
        assert len(s2) == 3, "Expected 2 families and one task suite"
    assert len(defs) == 1, "Expected 1 suite"

    with Suite("s2") as s2:
        s2.add_family("f1")
        s2.add_task("t1")
        assert len(s2) == 2, "Expected 2 nodes family and task"

    with Family("f1") as f1:
        f1.add_family("f2")
        f1.add_task("t1")
        assert len(s2) == 2, "Expected 2 nodes family and task"

    # test `add_*()`, using the functional way

    defs1 = Defs()
    defs1.add_suite("s1").add_task("t1").add_variable("var", "v")
    defs1.add_suite("s2").add_family("f1").add_task("t1").add_variable("var", "v")
    defs1.add_suite("s3").add_family("f1").add_family("f2").add_task("t1").add_variable("var", "v")

    # test `add_*()` using context manager
    # # this is the same as above, to allow test the comparison

    with Defs() as defs2:
        with defs2.add_suite("s1") as s1:
            t1 = s1.add_task("t1")
            t1.add_variable("var", "v")
        with defs2.add_suite("s2") as s2:
            with s2.add_family("f1") as f1:
                with f1.add_task("t1") as t1:
                    t1.add_variable("var", "v")
        with defs2.add_suite("s3") as s3:
            with s3.add_family("f1") as f1:
                with f1.add_family("f2") as f2:
                    with f2.add_task("t1") as t1:
                        t1.add_variable("var", "v")

    assert defs1 == defs2, "expected defs to be the same"

    # add node using with, they should compare
    with Defs() as defs3:
        with defs3.add_suite("s1") as s1:
            s1 + Task('t1', var='v')
        with defs3.add_suite("s2") as s2:
            with s2.add_family("f1") as f1:
                with f1.add_task("t1") as t1:
                    t1 += Edit(var='v')
        with defs3.add_suite("s3") as s3:
            with s3.add_family("f1") as f1:
                with f1.add_family("f2") as f2:
                    with f2.add_task("t1") as t1:
                        t1 += Edit(var='v')

    assert defs1 == defs3, "expected defs to be the same"

    print("All tests pass")



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

# This test ensures that sorting ecflow objects is correct.

import unittest
from ecflow import *


class Test_sort(unittest.TestCase):

    def get_object_list(self, obj, name_list):
        if obj == 'Meter':
            return [Meter(name, 0, 100) for name in name_list]
        elif obj == 'Label':
            return [Label(name, 'value') for name in name_list]
        elif obj == 'Variable':
            return [Variable(name, 'value') for name in name_list]
        elif obj == 'Limit':
            return [Limit(name, 10) for name in name_list]
        else:
            return [eval(obj)(name) for name in name_list]

    def test_sort_objects(self):
        names = ['zz', 'ww', 'aa']
        object_list = ['Task', 'Family', 'Suite', 'Event', 'Meter', 'Label', 'Variable', 'Limit']
        for obj in object_list:
            self.assertEqual(sorted(self.get_object_list(obj, names)), self.get_object_list(obj, sorted(names)), " sort of " + obj + " failed")


class Test_sort_api(unittest.TestCase):
    def test_sort_attributes_api(self):
        nodes = [Defs(), Suite('s'), Family('f'), Task('t')]
        sort_attr = ['event', 'meter', 'label', 'variable', 'all']
        for attr in sort_attr:
            for node in nodes:
                node.sort_attributes(attr)
                node.sort_attributes(attr, True)
                node.sort_attributes(attr, True, ['/path', '/path2'])

        sort_attr = [AttrType.event, AttrType.meter, AttrType.label, AttrType.variable, AttrType.all]
        for attr in sort_attr:
            for node in nodes:
                node.sort_attributes(attr)
                node.sort_attributes(attr, True)
                node.sort_attributes(attr, True, ['/path', '/path2'])


# see ECFLOW-1642
class Test_ECFLOW_1642(unittest.TestCase):
    def get_events(self):
        return [Event(name) for name in
                "foo buzz bar google zulu mason".split()]

    def sorted_events(self):
        return [Event(name) for name in
                "bar buzz foo google mason zulu".split()]

    def expected_def(self):
        defs = Defs(Suite("mysuite",
                          Family("fam1", self.get_events()),
                          Family("fam2", self.sorted_events(),
                                 Task("task1", self.sorted_events()),
                                 ),
                          Task("task2", self.sorted_events()),
                          self.sorted_events()))
        return defs

    def defs_to_sort(self):
        defs = Defs(Suite("mysuite",
                          Family("fam1", self.get_events()),
                          Family("fam2", self.get_events(),
                                 Task("task1", self.get_events()),
                                 ),
                          Task("task2", self.get_events()),
                          self.get_events()))
        return defs

    def test_no_sort(self):
        defs = self.defs_to_sort()
        defs.sort_attributes('event', True, ['/mysuite/fam1'])
        self.assertEqual(defs, self.expected_def(), "Expected\n" + str(self.expected_def()) + "but found\n" + str(defs))

    def test_no_sort2(self):
        defs = self.defs_to_sort()
        defs.sort_attributes(AttrType.event, True, ['/mysuite/fam1'])
        self.assertEqual(defs, self.expected_def(), "Expected\n" + str(self.expected_def()) + "but found\n" + str(defs))


class Test_defs_sort(unittest.TestCase):
    def test_defs_sort_attributes(self):
        defs = Defs()
        defs.add_variable("ZFRED", "/tmp/")
        defs.add_variable("YECF_URL_CMD", "${BROWSER:=firefox} -new-tab %ECF_URL_BASE%/%ECF_URL%")
        defs.add_variable("XECF_URL_BASE", "http://www.ecmwf.int")
        defs.add_variable("AECF_URL", "publications/manuals/sms")
        assert len(list(defs.user_variables)) == 4, "Expected *user* 4 variable"

        # sort
        expected = ['AECF_URL', 'XECF_URL_BASE', 'YECF_URL_CMD', 'ZFRED']
        actual = []
        defs.sort_attributes("variable");
        for v in defs.user_variables: actual.append(v.name())
        assert actual == expected, "Attributes not sorted, expected:" + str(expected) + " but found:" + str(actual)

        expected = ['AECF_URL', 'XECF_URL_BASE', 'YECF_URL_CMD', 'ZFRED', 'ZZ']
        actual = []
        defs.add_variable("ZZ", "x")
        defs.sort_attributes(AttrType.variable);
        for v in defs.user_variables: actual.append(v.name())
        assert actual == expected, "Attributes not sorted, expected:" + str(expected) + " but found:" + str(actual)

        expected = ['AA', 'AECF_URL', 'XECF_URL_BASE', 'YECF_URL_CMD', 'ZFRED', 'ZZ']
        actual = []
        defs.add_variable("AA", "x")
        defs.sort_attributes("all");
        for v in defs.user_variables: actual.append(v.name())
        assert actual == expected, "Attributes not sorted, expected:" + str(expected) + " but found:" + str(actual)

        expected = ['AA', 'AECF_URL', 'BB', 'XECF_URL_BASE', 'YECF_URL_CMD', 'ZFRED', 'ZZ']
        actual = []
        defs.add_variable("BB", "x")
        defs.sort_attributes(AttrType.all);
        for v in defs.user_variables: actual.append(v.name())
        assert actual == expected, "Attributes not sorted, expected:" + str(expected) + " but found:" + str(actual)


if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



#!/usr/bin/env python3

import pathlib
from ecflow import Client

if __name__ == '__main__':

    base = pathlib.Path.home() / "course"

    try:
        ci = Client()

        print("[1] Load suite definition in 'test.def' into the server")
        ci.load(str(base / "test.def"))
        # line above reads suite definition from disk and loads into the server

    except RuntimeError as e:
        print("Failed:", e)



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

from ecflow import Alias, AttrType, Autocancel, CheckPt, ChildCmdType, Client, Clock, Cron, DState, Date, Day, Days, \
    Defs, Ecf, Event, Expression, Family, FamilyVec, File, Flag, FlagType, FlagTypeVec, InLimit, \
    JobCreationCtrl, Label, Late, Limit, Meter, Node, NodeContainer, NodeVec, PartExpression, PrintStyle, \
    Repeat, RepeatDate, RepeatDateTime, RepeatDay, RepeatEnumerated, RepeatInteger, RepeatString, SState, State, Style, \
    Submittable, Suite, SuiteVec, Task, TaskVec, Time, TimeSeries, TimeSlot, Today, UrlCmd, Variable, \
    VariableList, Verify, WhyCmd, ZombieAttr, ZombieType, ZombieUserActionType, Trigger, Complete, Edit, Defstatus
import os
import unittest


class TestAddSuiteFamilyTask0(unittest.TestCase):
    def setUp(self):
        defs = Defs()
        s = Suite('s1')
        f = Family('f1')
        t = Task('t1')
        defs.add_suite(s)
        s.add_family(f)
        f.add_task(t)
        self.defs = defs

    def test_0(self):
        defs = Defs()
        defs.add_suite('s1').add_family('f1').add_task('t1')
        self.assertEqual(self.defs, defs, "defs not equal")

    def test_1(self):
        defs = Defs().add(
            Suite('s1').add(
                Family('f1').add(
                    Task('t1'))))
        self.assertEqual(self.defs, defs, "defs not equal")

    def test_2(self):
        defs = Defs(
            Suite('s1',
                  Family('f1',
                         Task('t1'))))
        self.assertEqual(self.defs, defs, "defs not equal")

    def test_3(self):
        defs = Defs() + (Suite('s1') + (Family('f1') + Task('t1')))
        self.assertEqual(self.defs, defs, "defs not equal")

    def test_3(self):
        defs = Defs() + (Suite('s1') + (Family('f1') + Task('t1')))
        self.assertEqual(self.defs, defs, "defs not equal")


class TestAddSuiteFamilyTask(unittest.TestCase):
    def setUp(self):
        defs = Defs()  # create an empty definition
        suite = defs.add_suite("s1")  # create a suite and add it to the defs
        family = suite.add_family("f1")  # create a family and add it to suite
        for i in ["a", "b", "c"]:  # create task ta,tb,tc
            family.add_task("t" + i)  # create a task and add to family
        defs.save_as_defs("test.def")  # save defs to file "test.def"

        self.defs = defs

    def test_me(self):
        with Defs() as defs:
            with defs.add_suite("s1") as suite:
                with suite.add_family("f1") as family:
                    for i in ["a", "b", "c"]:
                        family.add_task("t" + i)
        defs.save_as_defs("test.def")

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(self.defs, defs, "defs not equal")

    def test_me2(self):
        defs = Defs().add(
            Suite("s1").add(
                Family("f1").add(
                    [Task("t{0}".format(t))
                     for t in ("a", "b", "c")])))
        defs.save_as_defs("test.def")  # save defs to file "test.def"

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(self.defs, defs, "defs not equal")

    def test_me3(self):
        defs = Defs()
        defs += Suite("s1")
        defs.s1 += Family("f1")
        defs.s1.f1 += [Task("t{0}".format(t))
                       for t in ("a", "b", "c")]
        defs.save_as_defs("test.def")  # save defs to file "test.def"

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(self.defs, defs, "defs not equal")

    def test_me4(self):
        defs = Defs(
            Suite("s1",
                  Family("f1",
                         [Task("t{0}".format(t)) for t in ("a", "b", "c")])))
        defs.save_as_defs("test.def")  # save defs to file "test.def"

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(self.defs, defs, "defs not equal")

    def test_me5(self):
        defs = Defs().add(
            [Suite("s{0}".format(i)).add(
                [Family("f{0}".format(i)).add(
                    [Task("t{0}".format(i)) for i in range(1, 6)])
                    for i in range(1, 7)])
                for i in range(1, 6)])
        assert len(defs) == 5, " expected 6 suites but found " + str(len(defs))
        for suites in defs:
            assert len(suites) == 6, " expected 6 families but found " + str(len(suites))
            for fam in suites:
                assert len(fam) == 5, " expected 6 tasks but found " + str(len(fam))

    def test_me6(self):
        defs = Defs(
            [Suite("s{0}".format(i),
                   [Family("f{0}".format(i),
                           [Task("t{0}".format(i)) for i in range(1, 6)])
                    for i in range(1, 7)])
             for i in range(1, 6)])
        assert len(defs) == 5, " expected 5 suites but found " + str(len(defs))
        for suites in defs:
            assert len(suites) == 6, " expected 6 families but found " + str(len(suites))
            for fam in suites:
                assert len(fam) == 5, " expected 5 tasks but found " + str(len(fam))

    def tearDown(self):
        unittest.TestCase.tearDown(self)
        try:
            os.remove("test.def")
        except:
            pass


class TestAddMeterEventLabel(unittest.TestCase):
    def setUp(self):
        defs = Defs()
        suite = Suite("s1")
        task = Task("t1")
        defs.add_suite(suite)
        suite.add_task(task)
        task.add_event(2)  # event reference with 2
        task.add_event("wow")  # event reference with name "wow"
        task.add_event(10, "Eventname2")  # event referenced with name "Eventname2"
        task.add_meter("metername3", 0, 100)  # name, min, max
        task.add_label("label_name4", "value")  # name, value

        self.defs = defs

    def test_alternative(self):
        defs = Defs().add(
            Suite("s1").add(
                Task("t1").add(
                    Event(2),
                    Event("wow"),
                    Event(10, "Eventname2"),
                    Meter("metername3", 0, 100),
                    Label("label_name4", "value"))))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative1(self):
        defs = Defs() + Suite("s1")
        defs.s1 += Task("t1") + Event(2) + Event("wow") + \
                   Event(10, "Eventname2") + Meter("metername3", 0, 100) + \
                   Label("label_name4", "value")

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative2(self):
        with Defs() as defs:
            with defs.add_suite("s1") as suite:
                with suite.add_task("t1") as t1:
                    t1 += [Event(2),
                           Event("wow"),
                           Event(10, "Eventname2"),
                           Meter("metername3", 0, 100),
                           Label("label_name4", "value")]

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative3(self):
        defs = Defs(
            Suite('s1',
                  Task("t1",
                       Event(2),
                       Event("wow"),
                       Event(10, "Eventname2"),
                       Meter("metername3", 0, 100),
                       Label("label_name4", "value"))))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")


class TestAddLimitInlimit(unittest.TestCase):

    def setUp(self):
        defs = Defs()
        s1 = defs.add_suite("s1")
        s1.add_limit("limitName4", 10)  # name, maximum token
        f1 = s1.add_family("f1")
        f1.add_inlimit("limitName4", "/s1/f1", 2)  # limit name, path to limit, tokens consumed
        for i in range(1, 4):
            f1.add_task("t{0}".format(i))

        self.defs = defs

    def test_alternative(self):
        defs = Defs().add(
            Suite("s1").add(
                Limit("limitName4", 10),
                Family("f1").add(
                    InLimit("limitName4", "/s1/f1", 2),
                    [Task("t{0}".format(t)) for t in range(1, 4)]
                )))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative1(self):
        defs = Defs() + Suite("s1")
        defs.s1 += [Limit("limitName4", 10), Family("f1")]
        defs.s1.f1 += [InLimit("limitName4", "/s1/f1", 2),
                       [Task("t{0}".format(t)) for t in range(1, 4)]]

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative2(self):
        with Defs() as defs:
            with defs.add_suite("s1") as s1:
                s1.add_limit("limitName4", 10)  # name, maximum token
                with s1.add_family("f1") as f1:
                    f1.add_inlimit("limitName4", "/s1/f1", 2)  # limit name, path to limit, tokens consumed
                    f1 += [Task("t{0}".format(t)) for t in range(1, 4)]

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative3(self):
        defs = Defs(
            Suite("s1",
                  Limit("limitName4", 10),  # name, maximum token
                  Family("f1",
                         InLimit("limitName4", "/s1/f1", 2),  # limit name, path to limit, tokens consumed
                         [Task("t{0}".format(t)) for t in range(1, 4)])))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")


class TestAddVariable(unittest.TestCase):

    def setUp(self):
        defs = Defs()
        s1 = defs.add_suite("s1")
        s1.add_variable("HELLO", "world")  # name, value
        s1.add_variable({"NAME": "value", "NAME2": "value2", "NAME3": "value3", "NAME4": 4})
        s1.add_variable(Variable("FRED", "bloggs"))
        s1.add_variable(Variable("BILL", "1"))

        defs.s1.sort_attributes("variable");  # sort since with dictionary order of addition is arbitary
        self.defs = defs

    def test_alternative0(self):
        defs = Defs(Suite("s1", HELLO="world", FRED="bloggs", BILL=1, NAME="value", NAME2="value2"))
        defs.s1.add_variable('NAME4', 4)
        defs.s1 += Edit(NAME3="value3")

        defs.s1.sort_attributes("variable");
        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative(self):
        defs = Defs().add(
            Suite("s1").add(
                Edit(HELLO="world", FRED="bloggs", BILL=1, NAME="value", NAME2="value2", NAME3="value3", NAME4=4)
            ))

        defs.s1.sort_attributes("variable");
        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative1(self):
        defs = Defs() + Suite("s1", Edit(HELLO="world"))
        defs.s1 += [Edit({"NAME": "value", "NAME2": "value2", "NAME3": "value3", "NAME4": 4}, BILL=1),
                    Edit(FRED="bloggs")]

        defs.s1.sort_attributes("variable");
        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative2(self):
        defs = Defs(
            Suite("s1",
                  {"HELLO": "world", "NAME": "value", "NAME2": "value2",
                   "NAME3": "value3", "NAME4": 4, "BILL": 1, "FRED": "bloggs"}))

        defs.s1.sort_attributes("variable");
        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")


class TestAddTrigger(unittest.TestCase):

    def setUp(self):
        defs = Defs()
        s1 = defs.add_suite("s1")
        t1 = s1.add_task("t1")
        t2 = s1.add_task("t2")
        t2.add_trigger("t1 == active and t3 == aborted")
        t2.add_complete("t3 == complete")
        t3 = s1.add_task("t3")
        self.defs = defs

    def test_alternative0(self):
        defs = Defs(
            Suite("s1",
                  Task("t1"),
                  Task("t2",
                       Trigger("t1 == active and t3 == aborted"),
                       Complete("t3 == complete")),
                  Task("t3")))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative(self):
        defs = Defs().add(
            Suite("s1").add(
                Task("t1"), Task("t2").add(
                    Trigger("t1 == active and t3 == aborted"),
                    Complete("t3 == complete")),
                Task("t3")))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative1(self):
        defs = Defs() + Suite("s1")
        defs.s1 += [Task("t{0}".format(i)) for i in range(1, 4)]
        defs.s1.t2 += [Trigger("t1 == active and t3 == aborted"),
                       Complete("t3 == complete")]

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")


class TestAddTaskChain(unittest.TestCase):

    def setUp(self):
        defs = Defs(
            Suite("s1",
                  Task("t1"),
                  Task("t2", Trigger("t1 == complete")),
                  Task("t3", Trigger("t2 == complete")),
                  Task("t4", Trigger("t3 == complete"))))
        self.defs = defs

    def test_alternative(self):
        defs = Defs() + Suite("s1")
        defs.s1 += [Task("t1"), Task("t2"),
                    Task("t3"), Task("t4")]
        defs.s1.t2 += Trigger(["t1"])
        defs.s1.t3 += Trigger(["t2"])
        defs.s1.t4 += Trigger(["t3"])

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative1(self):
        defs = Defs() + Suite("s1")
        defs.s1 += [Task("t1"), Task("t2"), Task("t3"), Task("t4")]
        defs.s1.t2 += Trigger([defs.s1.t1])
        defs.s1.t3 += Trigger([defs.s1.t2])
        defs.s1.t4 += Trigger([defs.s1.t3])

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative1(self):
        defs = Defs(Suite("s1"))
        defs.s1 >> Task("t1") >> Task("t2") >> Task("t3") >> Task("t4")

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")


class TestAddReverseTaskChain(unittest.TestCase):

    def setUp(self):
        defs = Defs(
            Suite("s1",
                  Task("t1", Trigger("t2 == complete")),
                  Task("t2", Trigger("t3 == complete")),
                  Task("t3", Trigger("t4 == complete")),
                  Task("t4")))
        self.defs = defs

    def test_alternative1(self):
        defs = Defs() + Suite("s1")
        defs.s1 << Task("t1") << Task("t2") << Task("t3") << Task("t4")

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")


class TestAddLargeTrigger(unittest.TestCase):

    def setUp(self):
        defs = Defs()
        s1 = defs.add_suite("s1")
        t1 = s1.add_task("t1")
        t2 = s1.add_task("t2")
        t3 = s1.add_task("t3")
        t3.add_part_trigger("t1 == complete")
        t3.add_part_trigger("t2 == active", True)  # here True means add as 'AND'
        t3.add_part_trigger("t2 == aborted", False)  # here False means add as 'OR'
        self.defs = defs

    def test_alternative0(self):
        defs = Defs(
            Suite("s1",
                  Task("t1"),
                  Task("t2"),
                  Task("t3",
                       Trigger("t1 == complete"),
                       Trigger("t2 == active"),
                       Trigger("t2 == aborted", False))))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative(self):
        defs = Defs().add(
            Suite("s1").add(
                Task("t1"),
                Task("t2"),
                Task("t3").add(
                    Trigger("t1 == complete"),
                    Trigger("t2 == active"),
                    Trigger("t2 == aborted", False),
                )
            ))
        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative1(self):
        defs = Defs() + Suite("s1")
        defs.s1 += [Task("t{0}".format(i)) for i in range(1, 4)]
        defs.s1.t3 += [Trigger("t1 == complete"),
                       Trigger("t2 == active"),
                       Trigger("t2 == aborted", False)]

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")


class TestAddTimeDependencies(unittest.TestCase):

    def setUp(self):
        defs = Defs()
        s1 = defs.add_suite("s1")
        t1 = s1.add_task("date")
        t1.add_date(1, 0, 0)  # first of every month and every year
        t1.add_date(Date("2.*.*"))  # second of every month and every year
        t1.add_date(28, 2, 2026)  # 28 february 2026

        t2 = s1.add_task("day")
        t2.add_day("monday")
        t2.add_day(Days.tuesday)

        t3 = s1.add_task("time")
        t3.add_time("+00:30")  # 30 minutes after suite has begun
        t3.add_time("+00:30 20:00 01:00")  # 00:30,01:30,02:30....07:30 after suite start

        t3.add_time(0, 59, True)  # 00:59 - 59 minutes past midnight
        t3.add_time(Time(TimeSlot(20, 10)))  # 20:10 - 10 minutes pas eight
        t3.add_time(Time(TimeSlot(20, 20), True))  # +20:20 - 20 minutes and 20 hours, after suite start

        start = TimeSlot(0, 0)
        finish = TimeSlot(23, 0)
        incr = TimeSlot(0, 30)
        time_series = TimeSeries(start, finish, incr, True)
        t3.add_time(Time(time_series))
        t3.add_time(Time(0, 10))  # 00:10
        t3.add_time(Time("+00:40"))  # +00:40
        t3.add_time(Time("+00:40 20:00 01:00"))  # 00:40,01:40,02:30...17:40 after suite start

        cron = Cron()
        cron.set_week_days([0, 1, 2, 3, 4, 5, 6])
        cron.set_days_of_month([1, 2, 3, 4, 5, 6])
        cron.set_months([1, 2, 3, 4, 5, 6])
        cron.set_time_series("+00:00 23:00 00:30")
        s1.add_task("cron").add_cron(cron);

        self.defs = defs

    def test_alternative0(self):
        start = TimeSlot(0, 0)
        finish = TimeSlot(23, 0)
        incr = TimeSlot(0, 30)
        time_series = TimeSeries(start, finish, incr, True)

        defs = Defs(
            Suite("s1",
                  Task("date",
                       Date(1, 0, 0),  # first of every month and every year
                       Date("2.*.*"),  # second of every month and every yea
                       Date(28, 2, 2026)),  # 28 february 2026
                  Task("day",
                       Day("monday"),
                       Day(Days.tuesday)),
                  Task("time",
                       Time("+00:30"),  # 30 minutes after suite has begun
                       Time("+00:30 20:00 01:00"),  # 00:30,01:30,02:30....07:30 after suite start
                       Time(0, 59, True),  # 00:59 - 59 minutes past midnight
                       Time(TimeSlot(20, 10)),  # 20:10 - 10 minutes pas eight
                       Time(TimeSlot(20, 20), True),  # +20:20 - 20 minutes and 20 hours, after suite start
                       Time(time_series),
                       Time(0, 10),
                       Time("+00:40"),
                       Time("+00:40 20:00 01:00")),
                  Task("cron",
                       Cron("+00:00 23:00 00:30", days_of_week=[0, 1, 2, 3, 4, 5, 6], days_of_month=[1, 2, 3, 4, 5, 6], months=[1, 2, 3, 4, 5, 6]))))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative(self):
        start = TimeSlot(0, 0)
        finish = TimeSlot(23, 0)
        incr = TimeSlot(0, 30)
        time_series = TimeSeries(start, finish, incr, True)

        cron = Cron()
        cron.set_week_days([0, 1, 2, 3, 4, 5, 6])
        cron.set_days_of_month([1, 2, 3, 4, 5, 6])
        cron.set_months([1, 2, 3, 4, 5, 6])
        cron.set_time_series("+00:00 23:00 00:30")

        defs = Defs().add(
            Suite("s1").add(
                Task("date").add(
                    Date(1, 0, 0),
                    Date("2.*.*"),
                    Date(28, 2, 2026)),
                Task("day").add(
                    Day("monday"),
                    Day(Days.tuesday)),
                Task("time").add(
                    Time("+00:30"),
                    Time("+00:30 20:00 01:00"),
                    Time(0, 59, True),
                    Time(TimeSlot(20, 10)),
                    Time(TimeSlot(20, 20), True),
                    Time(time_series),
                    Time(0, 10),
                    Time("+00:40"),
                    Time("+00:40 20:00 01:00")),
                Task("cron").add(
                    cron)))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative1(self):
        start = TimeSlot(0, 0)
        finish = TimeSlot(23, 0)
        incr = TimeSlot(0, 30)
        time_series = TimeSeries(start, finish, incr, True)

        defs = Defs() + (Suite("s1") + Task("date") + Task("day") + Task("time") + Task("cron"))
        defs.s1.date += [Date(1, 0, 0), Date("2.*.*"), Date(28, 2, 2026)]
        defs.s1.day += [Day("monday"), Day(Days.tuesday)]
        defs.s1.time += [Time("+00:30"), Time("+00:30 20:00 01:00"), Time(0, 59, True),
                         Time(TimeSlot(20, 10)), Time(TimeSlot(20, 20), True),
                         Time(time_series), Time(0, 10), Time("+00:40"),
                         Time("+00:40 20:00 01:00")]
        defs.s1.cron += Cron("+00:00 23:00 00:30",
                             days_of_week=[0, 1, 2, 3, 4, 5, 6],
                             days_of_month=[1, 2, 3, 4, 5, 6],
                             months=[1, 2, 3, 4, 5, 6])

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")


class TestAddDefStatus(unittest.TestCase):

    def setUp(self):
        defs = Defs()
        s1 = defs.add_suite("s1")
        s1.add_task("t1").add_defstatus(Defstatus("complete"))
        s1.add_task("t2").add_defstatus(DState.complete)

        self.defs = defs

    def test_alternative0(self):
        defs = Defs(
            Suite("s1",
                  Task("t1", Defstatus("complete")),
                  Task("t2", Defstatus(DState.complete))))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative(self):
        defs = Defs().add(
            Suite("s1").add(
                Task("t1").add(Defstatus("complete")),
                Task("t2").add(Defstatus(DState.complete))))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative1(self):
        defs = Defs() + (Suite("s1") + Task("t1") + Task("t2"))
        defs.s1.t1 += Defstatus("complete")
        defs.s1.t2 += Defstatus(DState.complete)

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")


class TestAddAutocancel(unittest.TestCase):

    def setUp(self):
        defs = Defs()
        s1 = defs.add_suite("s1")
        s1.add_task("t1").add_autocancel(3)  # delete task after 3 days after completion
        s1.add_task("t2").add_autocancel(1, 10, True)  # delete task 1hr 10 min after task completion
        s1.add_task("t3").add_autocancel(TimeSlot(2, 10), True)  # delete task 2hr 10 min after task completion
        s1.add_task("t4").add_autocancel(Autocancel(1))  # delete task after 1 day after task completion
        s1.add_task("t5").add_autocancel(Autocancel(18, 10, False))  # delete task at 6:10pm once it has completed
        s1.add_task("t6").add_autocancel(Autocancel(TimeSlot(2, 10), False))  # delete task at 2:10am once it has completed

        self.defs = defs

    def test_alternative0(self):
        defs = Defs(
            Suite("s1",
                  Task("t1", Autocancel(3)),  # delete task after 3 days after completion
                  Task("t2", Autocancel(1, 10, True)),  # delete task 1hr 10 min after task completion
                  Task("t3", Autocancel(TimeSlot(2, 10), True)),  # delete task 2hr 10 min after task completion
                  Task("t4", Autocancel(1)),  # delete task after 1 day after task completion
                  Task("t5", Autocancel(18, 10, False)),  # delete task at 6:10pm once it has completed
                  Task("t6", Autocancel(2, 10, False))))  # delete task at 2:10am once it has completed

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative(self):
        defs = Defs().add(
            Suite("s1").add(
                Task("t1").add(Autocancel(3)),
                Task("t2").add(Autocancel(1, 10, True)),
                Task("t3").add(Autocancel(TimeSlot(2, 10), True)),
                Task("t4").add(Autocancel(1)),
                Task("t5").add(Autocancel(18, 10, False)),
                Task("t6").add(Autocancel(2, 10, False))))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative1(self):
        defs = Defs() + (Suite("s1") + [Task("t{0}".format(i)) for i in range(1, 7)])
        defs.s1.t1 += Autocancel(3)
        defs.s1.t2 += Autocancel(1, 10, True)
        defs.s1.t3 += Autocancel(TimeSlot(2, 10), True)
        defs.s1.t4 += Autocancel(1)
        defs.s1.t5 += Autocancel(18, 10, False)
        defs.s1.t6 += Autocancel(2, 10, False)

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")


class TestAddRepeat(unittest.TestCase):
    def setUp(self):
        def add_tasks(fam):
            for i in range(1, 3):
                fam.add_task(Task("t{0}".format(i)))

        defs = Defs()
        s1 = defs.add_suite("s1")
        f1 = s1.add_family("f1")
        f1.add_repeat(RepeatDate("YMD", 20100111, 20100115, 2))
        add_tasks(f1)

        f2 = s1.add_family("f2")
        f2.add_repeat(RepeatInteger("count", 0, 100, 2))
        add_tasks(f2)

        f3 = s1.add_family("f3")
        f3.add_repeat(RepeatEnumerated("enum", ["red", "green", "blue"]))
        add_tasks(f3)

        f4 = s1.add_family("f4")
        f4.add_repeat(RepeatString("enum", ["a", "b", "c"]))
        add_tasks(f4)

        f5 = s1.add_family("f5")
        f5.add_repeat(RepeatDay(1))
        add_tasks(f5)

        f6 = s1.add_family("f6")
        f6.add_repeat(RepeatDateTime("DT", "20100111T000000", "20100115T000000", "48:00:00"))
        add_tasks(f6)

        self.defs = defs

    def test_alternative0(self):
        defs = Defs(
            Suite("s1",
                  Family("f1",
                         RepeatDate("YMD", 20100111, 20100115, 2),
                         [Task("t{0}".format(i)) for i in range(1, 3)]),
                  Family("f2",
                         RepeatInteger("count", 0, 100, 2),
                         [Task("t{0}".format(i)) for i in range(1, 3)]),
                  Family("f3",
                         RepeatEnumerated("enum", ["red", "green", "blue"]),
                         [Task("t{0}".format(i)) for i in range(1, 3)]),
                  Family("f4",
                         RepeatString("enum", ["a", "b", "c"]),
                         [Task("t{0}".format(i)) for i in range(1, 3)]),
                  Family("f5",
                         RepeatDay(1),
                         [Task("t{0}".format(i)) for i in range(1, 3)]),
                  Family("f6",
                         RepeatDateTime("DT", "20100111T000000", "20100115T000000", "48:00:00"),
                         [Task("t{0}".format(i)) for i in range(1, 3)])))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative1(self):
        defs = Defs().add(
            Suite("s1").add(
                Family("f1").add(
                    RepeatDate("YMD", 20100111, 20100115, 2),
                    [Task("t{0}".format(i)) for i in range(1, 3)]),
                Family("f2").add(
                    RepeatInteger("count", 0, 100, 2),
                    [Task("t{0}".format(i)) for i in range(1, 3)]),
                Family("f3").add(
                    RepeatEnumerated("enum", ["red", "green", "blue"]),
                    [Task("t{0}".format(i)) for i in range(1, 3)]),
                Family("f4").add(
                    RepeatString("enum", ["a", "b", "c"]),
                    [Task("t{0}".format(i)) for i in range(1, 3)]),
                Family("f5").add(
                    RepeatDay(1),
                    [Task("t{0}".format(i)) for i in range(1, 3)]),
                Family("f6").add(
                    RepeatDateTime("DT", "20100111T000000", "20100115T000000", "48:00:00"),
                    [Task("t{0}".format(i)) for i in range(1, 3)])))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_alternative2(self):
        defs = Defs() + Suite("s1")
        defs.s1 += [Family("f{0}".format(i)).add(
            [Task("t{0}".format(i)) for i in range(1, 3)])
            for i in range(1, 7)]
        defs.s1.f1 += RepeatDate("YMD", 20100111, 20100115, 2)
        defs.s1.f2 += RepeatInteger("count", 0, 100, 2)
        defs.s1.f3 += RepeatEnumerated("enum", ["red", "green", "blue"])
        defs.s1.f4 += RepeatString("enum", ["a", "b", "c"])
        defs.s1.f5 += RepeatDay(1)
        defs.s1.f6 += RepeatDateTime("DT", "20100111T000000", "20100115T000000", "48:00:00")

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")


class TestAddLate(unittest.TestCase):
    def setUp(self):
        defs = Defs()
        suite = defs.add_suite('s1')

        late = Late()
        late.submitted(20, 10)  # hour, min
        late.active(2, 10)  # hour, min
        late.complete(3, 10, True)  # hour, min, relative
        suite.add_task('t1').add_late(late)
        self.defs = defs

    def test_1(self):
        # Can also pass late into the Task constructor
        defs = Defs(
            Suite('s1',
                  Task('t1',
                       Late(submitted='20:10', active='02:10', complete='+03:10'))))

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")

    def test_2(self):
        # Can also pass late into the Task constructor
        defs = Defs() + (Suite('s1') + Task('t1'))
        defs.s1.t1 += Late(submitted='20:10', active='02:10', complete='+03:10')

        Ecf.set_debug_equality(True)
        equals = (self.defs == defs)
        Ecf.set_debug_equality(False)
        self.assertEqual(defs, self.defs, "expected defs to be the same")


class Deadlock(unittest.TestCase):
    def setUp(self):
        defs = Defs().add(
            Suite("dead_lock").add(
                Family("family").add(
                    Task("t1").add(Trigger("t2 == complete")),
                    Task("t2").add(Trigger("t1 == complete")))))

        self.defs = defs

    def test_me(self):
        defs = Defs(
            Suite("dead_lock",
                  Family('family',
                         Task('t1',
                              Trigger("t2 == complete")),
                         Task('t2',
                              Trigger("t1 == complete")))))

        self.assertEqual(defs, self.defs, "expected defs to be the same")


if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

import os
from ecflow import Defs, Limit, InLimit, Client, debug_build
import ecflow_test_util as Test

if __name__ == "__main__":
    Test.print_test_start(os.path.basename(__file__))

    defs = Defs()
    suite = defs.add_suite("s1");
    suite.add_task("t1").add_trigger("t2 == active)")
    theCheckValue = defs.check();
    print("Message: '" + theCheckValue + "'")
    assert len(theCheckValue) != 0, "Expected Error: triggers fail parse, miss-matched brackets in expression."

    # The number of tokens specified on the in-limit must be less than or equal to those specified on the LIMIT
    defs = Defs()
    suite = defs.add_suite("s1");
    suite.add_limit(Limit("disk", 50));
    family = suite.add_family("anon");
    family.add_inlimit(InLimit("disk", "/s1", 100))
    family.add_task("t1");
    theCheckValue = defs.check();
    print("Message: '" + theCheckValue + "'")
    assert len(theCheckValue) != 0, "Expected Error: since inlimit value('100') is greater than the LIMIT value('50')"

    # When a path is specified on the in-limit we search for the limit on that path, otherwise the extern's
    defs = Defs()
    suite = defs.add_suite("s1");
    family = suite.add_family("anon");
    family.add_inlimit(InLimit("disk", "/s1", 100))
    family.add_task("t1");
    theCheckValue = defs.check();
    print("Message: '" + theCheckValue + "'")
    assert len(theCheckValue) != 0, "Expected warning: since inlimit PATH(/s1:disk) reference a limit('disk') that does not exist"

    # Add as extern and repeat the check
    defs.add_extern("/s1:disk")
    assert len(defs.check()) == 0, "Expected no warnings, since extern specified: " + defs.check()

    # When no path is specified on the in-limit, we search for the limit up the node tree, otherwise the extern's
    defs = Defs()
    suite = defs.add_suite("s1");
    family = suite.add_family("anon");
    family.add_inlimit(InLimit("disk", "", 100))
    family.add_task("t1");
    theCheckValue = defs.check();
    print("Message: '" + theCheckValue + "'")
    assert len(theCheckValue) != 0, "Expected warning: since inlimit SHOULD reference a limit('disk') somewhere UP parent hierarchy"

    # Add as extern and repeat the check
    defs.add_extern("disk")
    assert len(defs.check()) == 0, "Expected no warnings, since extern specified"

    print("All Tests pass")



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

import os
from ecflow import Defs, Suite, Variable, Limit, InLimit, Task, PartExpression, \
    Event, Meter, Label, Queue, RepeatInteger, RepeatEnumerated, RepeatDate, RepeatDateTime, \
    RepeatDateList, RepeatString, TimeSlot, TimeSeries, Today, Time, Date, Day, Days, Cron, Autocancel, \
    Late, DState, Clock, ChildCmdType, ZombieType, ZombieAttr, ZombieUserActionType, Client, debug_build
import ecflow_test_util as Test

if __name__ == "__main__":

    Test.print_test_start(os.path.basename(__file__))

    #
    # Add Nodes functional way
    #
    defs = Defs()
    defs.add_suite("s1").add_task("t1").add_variable("var", "v")
    defs.add_suite("s2").add_family("f1").add_task("t1").add_variable("var", "v")
    defs.add_suite("s3").add_family("f1").add_family("f2").add_task("t1").add_variable("var", "v")
    assert len(defs) == 3, "Expected 3 suites"

    # add and delete variables
    a_dict = {"name": "value", "name2": "value2", "name3": "value3", "name4": "value4"}
    suite = Suite("s1")
    suite.add_variable(Variable("ECF_HOME", "'/tmp/'")) \
        .add_variable("Fred", 1) \
        .add_variable("ECF_URL_BASE", 'http://www.ecmwf.int') \
        .add_variable("ECF_URL", '"publications/manuals/sms"') \
        .add_variable(a_dict)
    print(suite)
    assert len(list(suite.variables)) == 8, "Expected 8 variable"
    suite.delete_variable("");
    assert len(list(suite.variables)) == 0, "Expected 0 variable since we should have deleted all"

    # add and delete limits
    the_limit = Limit("limitName1", 10)
    assert len(list(the_limit.node_paths())) == 0, "Expected nodes which have consumed a limit to be empty"
    suite.add_limit(Limit("limitName1", 10)) \
        .add_limit(Limit("limitName2", 10)) \
        .add_limit("limitName3", 10) \
        .add_limit("limitName4", 10)
    assert len(list(suite.limits)) == 4, "Expected 4 Limits"
    suite.delete_limit("");
    assert len(list(suite.limits)) == 0, "Expected 0 limits since we just deleted all of them"

    # add and delete in-limits
    suite.add_inlimit(InLimit("limitName1", "/s1/f1", 2, True)) \
        .add_inlimit(InLimit("limitName2", "/s1/f1", 2)) \
        .add_inlimit("limitName3", "/s1/f1", 2, True) \
        .add_inlimit("limitName4", "/s1/f1", 2)
    assert len(list(suite.inlimits)) == 4, "Expected 4 inLimits"
    suite.delete_inlimit("limitName1");
    assert len(list(suite.inlimits)) == 3, "Expected 3 inlimits since we just deleted one limitName1"
    suite.delete_inlimit("");
    assert len(list(suite.inlimits)) == 0, "Expected 0 inlimits since we just deleted all of them"

    # add and delete triggers and complete
    task = Task("task")
    task.add_trigger("t2 == active").add_complete("t2 == complete")
    assert task.get_complete(), "Expected complete"
    assert task.get_trigger(), "Expected trigger"
    task.delete_trigger();
    assert not task.get_trigger(), "Expected no trigger"
    task.delete_complete();
    assert not task.get_complete(), "Expected no complete"

    task.add_part_trigger(PartExpression("t1 == complete")) \
        .add_part_complete(PartExpression("t3 == complete")) \
        .add_part_complete(PartExpression("t4 == active", False))
    task.delete_trigger();
    assert not task.get_trigger(), "Expected no trigger"
    task.delete_complete();
    assert not task.get_complete(), "Expected no complete"

    task.add_part_trigger("t1 == complete") \
        .add_part_complete("t3 == complete") \
        .add_part_complete("t4 == active", False)  # for long and/or expressions, subsequent expr must be and/or
    task.delete_trigger();
    assert not task.get_trigger(), "Expected no trigger"
    task.delete_complete();
    assert not task.get_complete(), "Expected no complete"

    # add and delete events
    task.add_event(Event(1)).add_event(2).add_event(Event(10, "Eventname")).add_event(10, "Eventname2").add_event("fred")
    assert len(list(task.events)) == 5, "Expected 5 Events"
    task.delete_event("");
    assert len(list(task.events)) == 0, "Expected 0 Events"

    # add and delete meter
    task.add_meter(Meter("metername1", 0, 100, 50)) \
        .add_meter(Meter("metername2", 0, 100)) \
        .add_meter("metername3", 0, 100, 50) \
        .add_meter("metername4", 0, 100)
    assert len(list(task.meters)) == 4, "Expected 4 Meters"
    task.delete_meter("");
    assert len(list(task.meters)) == 0, "Expected 0 Meters"

    # add and delete queue
    queue_items = ["001", "002"]
    task.add_queue(Queue("q1", queue_items)) \
        .add_queue(Queue("q2", queue_items)) \
        .add_queue("q3", queue_items) \
        .add_queue("q4", queue_items)
    assert len(list(task.queues)) == 4, "Expected 4 Queues"
    task.delete_queue("");
    assert len(list(task.queues)) == 0, "Expected 0 Queues"

    # add and delete label
    task.add_label(Label("label_name1", "value")) \
        .add_label(Label("label_name2", "value")) \
        .add_label("label_name3", "value") \
        .add_label("label_name4", "value")
    assert len(list(task.labels)) == 4, "Expected 4 labels"
    task.delete_label("");
    assert len(list(task.labels)) == 0, "Expected 0 Labels"

    # add delete Repeat
    task.add_repeat(RepeatInteger("integer", 0, 100, 2)).add_variable("fred", "j")
    repeat = task.get_repeat();
    assert not repeat.empty(), "Expected repeat"
    task.delete_repeat()
    repeat = task.get_repeat();
    assert repeat.empty(), "Expected no repeat"

    task.add_repeat(RepeatEnumerated("enum", ["red", "green", "blue"])).add_variable("Q", "j")
    print(task)
    task.delete_repeat()
    repeat = task.get_repeat();
    assert repeat.empty(), "Expected no repeat"

    task.add_repeat(RepeatDate("date", 20100111, 20100115, 2)).add_variable("W", "j")
    task.delete_repeat()
    repeat = task.get_repeat();
    assert repeat.empty(), "Expected no repeat"

    task.add_repeat(RepeatDateTime("datetime", "20100111T000000", "20100115T000000", "48:00:00")).add_variable("W", "j")
    task.delete_repeat()
    repeat = task.get_repeat();
    assert repeat.empty(), "Expected no repeat"

    task.add_repeat(RepeatDateList("date", [20100111, 20100115])).add_variable("W", "j")
    task.delete_repeat()
    repeat = task.get_repeat();
    assert repeat.empty(), "Expected no repeat"

    task.add_repeat(RepeatString("string", ["a", "b", "c"])).add_variable("E", "j")
    task.delete_repeat()
    repeat = task.get_repeat();
    assert repeat.empty(), "Expected no repeat"

    # create a time series, used for adding time and today
    start = TimeSlot(0, 0)
    finish = TimeSlot(23, 0)
    incr = TimeSlot(0, 30)
    time_series = TimeSeries(start, finish, incr, True)

    # Add and delete today
    # NOTE: **********************************************************************
    # Do *NOT* delete attributes using iterator traversal
    # The iterators *are* bound to the c++ iterators hence if we delete
    # over the traversal, we can corrupt the vector, leading to undefined behaviour.  
    # Hence we **cannot** delete more than once over a traversal
    # Soln 1: take a copy
    # *****************************************************************************
    # task.add_today( Today( 0,10 ))
    # task.add_today( Today( 0,59, True ))
    # task.add_today( Today(TimeSlot(20,10)) )
    # task.add_today( Today(TimeSlot(20,20),False)) 
    # for today in task.todays: 
    #    task.delete_today(today) # will corrupt C++ vector

    task.add_today("00:30").add_today("+00:30").add_today("+00:30 20:00 01:00") \
        .add_today(Today(time_series)) \
        .add_today(Today(0, 10)).add_today(0, 59, True) \
        .add_today(Today(TimeSlot(20, 10))) \
        .add_today(Today(TimeSlot(20, 20), False))
    assert len(list(task.todays)) == 8, "Expected 8 todays"
    vec_copy = []
    for today in task.todays: vec_copy.append(today)
    for today in vec_copy: task.delete_today(today)
    assert len(list(task.todays)) == 0, "Expected 0 todays"

    # add and delete time
    task.add_time("00:30").add_time("+00:30").add_time("+00:30 20:00 01:00") \
        .add_time(Time(time_series)) \
        .add_time(Time(0, 10)) \
        .add_time(0, 59, True) \
        .add_time(Time(TimeSlot(20, 10))) \
        .add_time(Time(TimeSlot(20, 20), False))
    assert len(list(task.times)) == 8, "Expected 8 times"
    vec_copy = []
    for time in task.times: vec_copy.append(time)
    for time in vec_copy: task.delete_time(time)
    assert len(list(task.todays)) == 0, "Expected 0 todays"

    # add and delete date
    for i in [1, 2, 4, 8, 16]:
        task.add_date(i, 0, 0)
    task.add_date(Date(1, 1, 2010)) \
        .add_date(Date(2, 1, 2010)) \
        .add_date(Date(3, 1, 2010)) \
        .add_date(Date(4, 1, 2010)) \
        .add_date(1, 1, 2010)
    assert len(list(task.dates)) == 10, "Expected 10 dates but found " + str(len(list(task.dates)))
    vec_copy = []
    for attr in task.dates: vec_copy.append(attr)
    for attr in vec_copy: task.delete_date(attr)
    assert len(list(task.dates)) == 0, "Expected 0 dates"

    # add and delete day
    task.add_day(Day(Days.sunday)) \
        .add_day(Days.monday) \
        .add_day(Days.tuesday) \
        .add_day("sunday")
    assert len(list(task.days)) == 4, "Expected 4 days"
    vec_copy = []
    for attr in task.days: vec_copy.append(attr)
    for attr in vec_copy: task.delete_day(attr)
    assert len(list(task.days)) == 0, "Expected 0 days"

    # add and delete crons
    cron = Cron()
    cron.set_week_days([0, 1, 2, 3, 4, 5, 6])
    cron.set_days_of_month([1, 2, 3, 4, 5, 6])
    cron.set_months([1, 2, 3, 4, 5, 6])
    start = TimeSlot(0, 0)
    finish = TimeSlot(23, 0)
    incr = TimeSlot(0, 30)
    ts = TimeSeries(start, finish, incr, True)  # True means relative to suite start
    cron.set_time_series(ts)

    cron1 = Cron()
    cron1.set_week_days([0, 1, 2, 3, 4, 5, 6])
    cron1.set_time_series(1, 30, True)

    cron2 = Cron()
    cron2.set_week_days([0, 1, 2, 3, 4, 5, 6])
    cron2.set_time_series("00:30 01:30 00:01")

    cron3 = Cron()
    cron3.set_week_days([0, 1, 2, 3, 4, 5, 6])
    cron3.set_time_series("+00:30")

    task.add_cron(cron) \
        .add_cron(cron1) \
        .add_cron(cron2) \
        .add_cron(cron3)
    assert len(list(task.crons)) == 4, "Expected 4 crons"
    vec_copy = []
    for attr in task.crons: vec_copy.append(attr)
    for attr in vec_copy: task.delete_cron(attr)
    assert len(list(task.crons)) == 0, "Expected 0 crons"

    # add autocancel
    t1 = Task("t1")
    t1.add_autocancel(3).add_variable("A", "j")
    t3 = Task("t3")
    t3.add_autocancel(20, 10, True).add_variable("B", "j")
    t4 = Task("t4")
    t4.add_autocancel(TimeSlot(10, 10), True).add_variable("C", "j")
    t5 = Task("t5")
    t5.add_autocancel(Autocancel(1, 10, True)).add_variable("D", "j")

    # add late
    late = Late()
    late.submitted(TimeSlot(20, 10))
    late.active(TimeSlot(20, 10))
    late.complete(TimeSlot(20, 10), True)
    task.add_late(late).add_variable("FRED33", "j")

    # add defstatus, last one set takes effect
    task.add_defstatus(DState.complete) \
        .add_defstatus(DState.queued) \
        .add_defstatus(DState.aborted) \
        .add_defstatus(DState.submitted) \
        .add_defstatus(DState.suspended) \
        .add_defstatus(DState.active)

    #
    # add clock
    #
    clock = Clock(1, 1, 2010, False)  # day,month, year, hybrid
    clock.set_gain(1, 10, True)  # True means positive gain
    suite = Suite("suite")
    suite.add_clock(clock).add_variable("fred1", "j")

    end_clock = Clock(1, 1, 2017, False)  # day,month, year, hybrid
    suite.add_end_clock(end_clock).add_variable("fred2", "j")

    clock = Clock(1, 1, 2011, True)  # day,month, year, hybrid
    clock.set_gain_in_seconds(12, True)
    s1 = Suite("s1")
    s1.add_clock(clock).add_variable("fred3", "j")
    s1.add_end_clock(end_clock).add_variable("fred4", "j")

    #
    # Add zombie. Note we can *NOT* add two zombie attributes of the same ZombieType
    #
    zombie_life_time_in_server = 800
    child_list = [ChildCmdType.init, ChildCmdType.event, ChildCmdType.meter, ChildCmdType.label, ChildCmdType.wait, ChildCmdType.abort, ChildCmdType.complete]
    zombie_type_list = [ZombieType.ecf, ZombieType.user, ZombieType.path]
    count = 1;
    for zombie_type in zombie_type_list:
        zombie_attr = ZombieAttr(zombie_type, child_list, ZombieUserActionType.block, zombie_life_time_in_server)
        s1.add_zombie(zombie_attr).add_variable("afred" + str(count), "j")
        count += 1
    assert len(list(s1.zombies)) == 3, "Expected 3 zombie attributes but found " + str(len(list(s1.zombies)))

    # delete all the zombies
    s1.delete_zombie("")
    assert len(list(s1.zombies)) == 0, "Expected zero zombie attributes but found " + str(len(list(s1.zombies)))

    # repeat the the test with empty child list. Empty child list means apply to all child commands
    # + don't specify zombie_life_time_in_server as this is optional
    child_list = []
    zombie_type_list = [ZombieType.ecf, ZombieType.user, ZombieType.path]
    for zombie_type in zombie_type_list:
        zombie_attr = ZombieAttr(zombie_type, child_list, ZombieUserActionType.block)
        s1.add_zombie(zombie_attr).add_variable("bfred" + str(count), "j")

        # test delete of specific zombie attribute
        assert len(list(s1.zombies)) == 1, "Expected 1 zombie attributes but found " + str(len(list(s1.zombies)))
        s1.delete_zombie(zombie_type)
        assert len(list(s1.zombies)) == 0, "Expected 0 zombie attributes but found " + str(len(list(s1.zombies)))
        count += 1;

    print("All Tests pass")



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

from platform import python_version
from socket import gethostname
import os, sys, fnmatch
import time
import fcntl
import datetime, time
import shutil  # used to remove directory tree
import platform  # for python version info
import inspect
import pathlib
from enum import Enum

import ecflow as ecf


def all_files(root, patterns='*', single_level=False, yield_folders=False):
    """Expand patterns from semi-colon separated string to list"""
    patterns = patterns.split(';')
    for path, subdirs, files in os.walk(root):
        if yield_folders:
            files.extend(subdirs)
        files.sort()
        for name in files:
            for pattern in patterns:
                if fnmatch.fnmatch(name, pattern):
                    yield os.path.join(path, name)
                    break
        if single_level:
            break

        # Enable to stop data being deleted, and stop server from being terminated


def debugging(): return False


def ecf_home(port):
    # Allowing multiple invocations of these tests, by creating a unique ECF_HOME based on:
    #  - python version
    #  - process id
    #  - build_type
    #  - server port,
    python_version = sys.version_info[0]
    build_type = "debug" if ecf.debug_build() else "release"
    pid = os.getpid()

    ecf_home_dir = f"ecf_home_py{python_version}__pid_{pid}__build_{build_type}__port_{port}"

    ecf_home_path = pathlib.Path.joinpath(pathlib.Path.cwd(), "test", "data", ecf_home_dir)

    return str(ecf_home_path.absolute())


def get_parent_dir(file_path):
    return os.path.dirname(file_path)


def log_file_path(port): return "./" + gethostname() + "." + port + ".ecf.log"


def checkpt_file_path(port): return "./" + gethostname() + "." + port + ".ecf.check"


def backup_checkpt_file_path(port): return "./" + gethostname() + "." + port + ".ecf.check.b"


def white_list_file_path(port): return "./" + gethostname() + "." + port + ".ecf.lists"


def print_test_start(test_name):
    print("#######################################################################################")
    print(test_name)
    print("ecflow version(" + ecf.Client().version() + ") debug build(" + str(ecf.debug_build()) + ")  python(" + platform.python_version() + ")")
    print("PYTHONPATH                : " + str(os.environ['PYTHONPATH'].split(os.pathsep)))
    # print("sys.path                  : " + str(sys.path))
    print("Current working directory : " + str(os.getcwd()))
    print("Python version            : " + str(sys.version_info[0]) + "." + str(sys.version_info[1]))
    print("#######################################################################################")


def clean_up_server(port):
    print("   clean_up " + port)
    try:
        os.remove(log_file_path(port))
    except:
        pass
    try:
        os.remove(checkpt_file_path(port))
    except:
        pass
    try:
        os.remove(backup_checkpt_file_path(port))
    except:
        pass
    try:
        os.remove(white_list_file_path(port))
    except:
        pass


def clean_up_data(port):
    print("   Attempting to Removing ECF_HOME " + ecf_home(port))
    try:
        shutil.rmtree(ecf_home(port), True)  # True means ignore errors
        print("   Remove OK")
    except:
        print("   Remove Failed")
        pass


class Protocol(Enum):
    CUSTOM = 1  # uses custom TCP/IP access (default)
    HTTP = 2  # uses HTTP access


# =======================================================================================
class EcfPortLock(object):
    """allow debug and release version of python tests to run at the same
    time, buy generating a unique port each time"""

    def __init__(self, protocol):
        print("   EcfPortLock:__init__")
        self._protocol = protocol
        pass

    def at_time(self):
        return datetime.datetime.fromtimestamp(time.time()).strftime('%H:%M:%S')

    def find_free_port(self, seed_port):
        print("   EcfPortLock:find_free_port starting with " + str(seed_port) + " at time " + self.at_time())
        port = seed_port
        while 1:
            # port must be free for at least 15 seconds
            if self._timed_free_port(port, 3) == True:
                print("   *FOUND* free server port " + str(port) + " : " + self.at_time())
                if self.do_lock(port):
                    break
            else:
                print("   *Server* port " + str(port) + " busy( by  ping), trying next port " + self.at_time())
            port = port + 1

        return str(port)

    def _timed_free_port(self, port, wait_time=10):
        count = 0
        while count < wait_time:
            if self._free_port(port) == True:
                count = count + 1
                time.sleep(1)
            else:
                return False
        return self._free_port(port)

    def _free_port(self, port):
        try:
            ci = ecf.Client()
            if self._protocol == Protocol.HTTP:
                ci.enable_http()
            ci.set_host_port("localhost", str(port))
            ci.ping()
            return False
        except RuntimeError as e:
            return True

    def do_lock(self, port):
        file = self._lock_file(port)
        if os.path.exists(file):
            print("   *LOCKED* lock file exists " + file + " trying next port : " + self.at_time())
            return False
        try:
            fp = open(file, 'w')
            try:
                self.lock_time = self.at_time()
                fcntl.lockf(fp, fcntl.LOCK_EX | fcntl.LOCK_NB)
                self.lock_file_fp = fp
                print("   *LOCKED* file " + file + " : " + self.lock_time)
                return True;
            except IOError:
                print("   Could *NOT* lock file " + file + " trying next port : " + self.at_time())
                return False
        except IOError as e:
            print("   Could not open file " + file + " for write trying next port : " + self.at_time())
            return False

    def remove(self, port):
        file = self._lock_file(port)
        print("   Remove lock file : " + file + " : lock_time: " + self.lock_time + " release_time: " + self.at_time())
        self.lock_file_fp.close()
        os.remove(file)

    def _lock_file(self, port):
        if "ECF_PORT_LOCK_DIR" in os.environ:
            lock_file = os.environ["ECF_PORT_LOCK_DIR"] + "/" + str(port) + ".lock"
            # print("     EcfPortLock::_lock_file ECF_PORT_LOCK_DIR: " + lock_file)
            return lock_file
        if os.path.exists(ecf.File.source_dir()):
            lock_file = ecf.File.source_dir() + "/" + str(port) + ".lock"
            return lock_file
        lock_file = os.getcwd() + "/" + str(port) + ".lock"
        return lock_file


# ===============================================================================

class Server(object):
    """TestServer: allow debug and release version of python tests to run at the same
    time, by generating a unique port each time"""

    def __init__(self, protocol=Protocol.CUSTOM):
        print("Server:__init__: Starting server")
        self._protocol = protocol
        if not debugging():
            seed_port = int(os.getenv('ECF_FREE_PORT', '3153'))
            if sys.version_info[0] == 3:  # python3 can run at same time
                seed_port = int(os.getenv('ECF_FREE_PORT', '3154'))
            if ecf.debug_build(): seed_port = seed_port + 1
            self.lock_file = EcfPortLock(self._protocol)
            self.the_port = self.lock_file.find_free_port(seed_port)
        else:
            self.the_port = "3152"

        # Only worth doing this test, if the server is running
        # ON HPUX, having only one connection attempt, sometimes fails
        # ci.set_connection_attempts(1)     # improve responsiveness only make 1 attempt to connect to server
        # ci.set_retry_connection_period(0) # Only applicable when make more than one attempt. Added to check api.
        self.ci = ecf.Client("localhost", self.the_port)
        if self._protocol == Protocol.HTTP:
            self.ci.enable_http()

    def at_time(self):
        return datetime.datetime.fromtimestamp(time.time()).strftime('%Y-%m-%d %H:%M:%S')

    def __enter__(self):
        try:
            print("Server:__enter__: About to ping localhost: " + self.the_port + " : " + self.at_time())
            self.ci.ping()
            print("   ------- Server all ready running on port " + self.the_port + " *UNEXPECTED* -- : " + self.at_time())
            sys.exit(1)
        except RuntimeError as e:
            while 1:
                # Only worth doing this test, if the server is running
                # ON HPUX, having only one connection attempt, sometimes fails
                # ci.set_connection_attempts(1)     # improve responsiveness only make 1 attempt to connect to server
                # ci.set_retry_connection_period(0) # Only applicable when make more than one attempt. Added to check api.
                self.ci = ecf.Client("localhost", self.the_port)
                if self._protocol == Protocol.HTTP:
                    self.ci.enable_http()

                print("   ------- Server *NOT* running on port " + self.the_port + " as *EXPECTED* ------ ")
                print("   ------- Start the server on port " + self.the_port + " ---------")
                if not debugging():
                    clean_up_server(str(self.the_port))
                    clean_up_data(str(self.the_port))

                server_exe = ecf.File.find_server();
                assert len(server_exe) != 0, "Could not locate the server executable"

                server_exe += " --port=" + self.the_port + " --ecfinterval=4"
                if self._protocol == Protocol.HTTP:
                    server_exe += " --http"
                server_exe += " &"
                print("   TestClient.py: Starting server " + server_exe)
                os.system(server_exe)

                print("   Allow time for server to start " + self.at_time())
                if self.ci.wait_for_server_reply():
                    print("   Server has started " + self.at_time())
                    break
                else:
                    print("   Server failed to start after 60 second, trying next port !!!!!! " + self.at_time())
                    self.the_port = self.lock_file.find_free_port(int(self.the_port) + 1)

        print("   Run the tests, leaving Server:__enter__:")

        # return the Client, that can call to the server
        return [self.ci, self._protocol]

    def __exit__(self, exctype, value, tb):
        print("Server:__exit__: Kill the server, clean up log file, check pt files and lock files, ECF_HOME " + self.at_time())
        print("   exctype:", exctype)
        print("   value:", value)
        print("   tb:", tb)
        print("   Terminate server " + self.at_time())
        self.ci.terminate_server()
        print("   Terminate server OK " + self.at_time())
        self.lock_file.remove(self.the_port)

        # Do not clean up data, if an assert was raised. This allow debug
        if exctype is None:
            if not debugging():
                clean_up_server(str(self.the_port))
                clean_up_data(str(self.the_port))
        return False


# ===============================================================================
class MockEnvironment:
    def __init__(self, env):
        self.expected_env = env

    def __enter__(self):
        self.original_env = {};
        for (name, value) in self.expected_env.items():
            if name in os.environ:
                self.original_env[name] = os.environ[name]
            else:
                self.original_env[name] = None
            os.environ[name] = value

    def __exit__(self, exception_type, exception_value, exception_traceback):
        for (name, value) in self.original_env.items():
            if value is None:
                del os.environ[name]
            else:
                os.environ[name] = value

    def __str__(self):
        return f"Test.MockEnvironment({self.expected_env})"


# ===============================================================================
class MockFile:
    def __init__(self, fname, fcontent=""):
        self.name = fname
        self.content = fcontent

    def __enter__(self):
        with open(self.name, "w") as file:
            file.write(self.content)

    def __exit__(self, exception_type, exception_value, exception_traceback):
        os.remove(self.name)
        pass

    def __str__(self):
        return f"Test.MockFile({self.name}, {self.content})"


# ==============================================================================
def name_this_test():
    """Set the name of the test based on the current function name"""
    test_name = inspect.stack()[1].function
    print(f"Test: {test_name}")


# ==============================================================================
def test_started():
    print_test_start(inspect.stack()[1].filename)


# ==============================================================================
def test_success():
    print("#######################################################################################")
    print(" All Tests passed")
    print("#######################################################################################")


def execute_all(clazz):
    test_started()

    for name, test in inspect.getmembers(clazz):
        if inspect.isfunction(test):
            test()

    test_success()



#!/usr/bin/env python3

import pathlib
from ecflow import Client

if __name__ == '__main__':

    base = pathlib.Path.home() / "course"

    try:
        ci = Client()

        print("[1] Syncing local defs with server")
        ci.sync_local()

        print("[2] Access the suite definition from the server")
        defs = ci.get_defs()

        print("[3] Restarting the server")
        if len(defs) == 0:
            print("No suites in server, loading defs from disk")
            ci.load(str(base / "test.def"))

            print("Restarting the server. This starts job scheduling")
            ci.restart_server()
        else:
            print("read definition from disk and replace on the server")
            ci.replace("/test", str(base / "test.def"))

        print("[4] Begin the suite")
        ci.begin_suite("test")

    except RuntimeError as e:
        print("Failed:", e)



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

import os

# clean up any file after running the online tutorial examples
# Place in separate exception, otherwise not all files will be removed
try:
    os.remove("test.def")
except:
    pass

try:
    os.remove("TestSuite.def")
except:
    pass

try:
    os.remove("SuiteBuilder.def")
except:
    pass

try:
    os.remove("server.defs")
except:
    pass



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

import os
from ecflow import Day, Date, Meter, Event, Queue, Clock, Variable, Label, Limit, InLimit, \
    RepeatDate, RepeatDateTime, RepeatEnumerated, RepeatInteger, RepeatString, \
    Task, Family, Suite, Defs, Client, debug_build, Trigger
import ecflow_test_util as Test


def check_day(day):
    try:
        Day(day)
        return True
    except RuntimeError:
        return False


def check_date(day, month, year):
    try:
        Date(day, month, year)
        return True
    except IndexError:
        return False


def check_date_str(str_date):
    try:
        Date(str_date)
        return True
    except IndexError:
        return False
    except  RuntimeError:
        return False


def check_meter(name, min_meter_value, max_meter_value, color_change):
    try:
        Meter(name, min_meter_value, max_meter_value, color_change)
        return True
    except IndexError:
        return False
    except RuntimeError:
        return False


def check_queue(name, queue_items):
    try:
        Queue(name, queue_items)
        return True
    except IndexError:
        return False
    except RuntimeError:
        return False


def check_event_number_and_name(number, name):
    try:
        Event(number, name)
        return True
    except:
        return False


def check_event(number):
    try:
        Event(number)
        return True
    except RuntimeError:
        return False


def check_clock(day_of_month, month, year):
    try:
        Clock(day_of_month, month, year)
        return True
    except IndexError:
        return False


def check_variable(name, value):
    try:
        Variable(name, value)
        return True
    except RuntimeError:
        return False


def check_label(name, value):
    try:
        Label(name, value)
        return True
    except RuntimeError:
        return False


def check_limit(name, int_token):
    try:
        Limit(name, int_token)
        return True
    except RuntimeError:
        return False
    except TypeError:
        return False


def check_inlimit(name, path_to_node, int_token):
    try:
        InLimit(name, path_to_node, int_token)
        return True
    except RuntimeError:
        return False
    except TypeError:
        return False


def check_repeat_date(name, start, end, step):
    try:
        RepeatDate(name, start, end, step)
        return True
    except RuntimeError:
        return False


def check_repeat_datetime(name, start, end, step):
    try:
        RepeatDateTime(name, start, end, step)
        return True
    except RuntimeError:
        return False


def check_repeat_integer(name, start, end, step):
    try:
        RepeatInteger(name, start, end, step)
        return True
    except RuntimeError:
        return False
    except TypeError:
        return False


def check_repeat_enumerated(name, list_of_strings):
    try:
        RepeatEnumerated(name, list_of_strings)
        return True
    except RuntimeError:
        return False
    except TypeError:
        return False


def check_repeat_string(name, list_of_strings):
    try:
        RepeatString(name, list_of_strings)
        return True
    except RuntimeError:
        return False
    except TypeError:
        return False


def check_node_name(name):
    try:
        Task(name)
        Family(name)
        Suite(name)
        return True;
    except RuntimeError:
        return False


def check_defs(path_to_defs):
    try:
        Defs(path_to_defs)
        return True
    except RuntimeError:
        return False


if __name__ == "__main__":
    Test.print_test_start(os.path.basename(__file__))

    # Names with leading '.' should not be allowed. Will interfere with triggers
    # Empty names not allowed
    # Spaces not allowed
    invalid_names = [".", "", " ", "   ", "fred doc", "1 "]

    # Allow names with leading underscore
    valid_names = ["_", "__", "_._", "1.2", "fred.doc", "_.1"]

    assert check_day("monday"), "Expected valid day"
    assert check_day("tuesday"), "Expected valid day"
    assert check_day("wednesday"), "Expected valid day"
    assert check_day("thursday"), "Expected valid day"
    assert check_day("friday"), "Expected valid day"
    assert check_day("saturday"), "Expected valid day"
    assert check_day("sunday"), "Expected valid day"
    assert check_day("") == False, "Expected exeception"
    assert check_day("sunday1") == False, "Expected exeception"
    assert check_day("2") == False, "Expected exeception"

    assert check_date(0, 1, 2010), "Expected valid date"
    assert check_date(10, 0, 2010), "Expected valid date"
    assert check_date(10, 1, 0), "Expected valid date"
    assert check_date(0, 0, 0), "Expected valid date"
    assert check_date(40, 1, 2010) == False, "Expected exception since day > 31"
    assert check_date(-10, 1, 2010) == False, "Expected exception since day >= 0"
    assert check_date(1, 14, 2010) == False, "Expected exception since month > 12"
    assert check_date(1, -1, 2010) == False, "Expected exception since month >= 0"
    assert check_date(1, 1, -2) == False, "Expected exception since year >= 0"

    assert check_date_str("*.1.2010"), "Expected valid date"
    assert check_date_str("10.*.2010"), "Expected valid date"
    assert check_date_str("10.1.*"), "Expected valid date"
    assert check_date_str("*.*.*"), "Expected valid date"
    assert check_date_str("40.1.2010") == False, "Expected exception since day > 31"
    assert check_date_str("-10.1.2010") == False, "Expected exception since day >= 0"
    assert check_date_str("1.14.2010") == False, "Expected exception since month > 12"
    assert check_date_str("1.-1.2010") == False, "Expected exception since month >= 0"
    assert check_date_str("1.1.-2") == False, "Expected exception since year >= 0"

    # clock do not support wild carding hence we cant use 0 like in Date
    assert check_clock(12, 1, 2010), "Expected valid date"
    assert check_clock(10, 1, 2010), "Expected valid date"
    assert check_clock(10, 1, 1400), "Expected valid date"
    assert check_clock(31, 12, 2010), "Expected valid date"
    assert check_clock(40, 1, 2010) == False, "Expected exception since day > 31"
    assert check_clock(-10, 1, 2010) == False, "Expected exception since day >= 0"
    assert check_clock(1, 14, 2010) == False, "Expected exception since month > 12"
    assert check_clock(1, -1, 2010) == False, "Expected exception since month >= 0"
    assert check_clock(1, 1, -2) == False, "Expected exception since year >= 0"

    assert check_meter("m", 0, 100, 100), "Expected valid Meter"
    assert check_meter("m", 0, 100, 0), "Expected valid Meter"
    assert check_meter("m", 200, 100, 100) == False, "Expected exception since min > max"
    assert check_meter("m", 0, 100, -20) == False, "Expected exception since color_change should between min-max"
    assert check_meter("m", 0, 100, 200) == False, "Expected exception since color_change should between min-max"
    assert check_meter("", 0, 100, 100) == False, "Expected exception since no name specified"
    assert check_meter(" ", 0, 100, 100) == False, "Expected Exception cannot have spaces for a name"

    assert check_queue("m", ["a"]), "Expected valid Queue"
    assert check_queue("m", ["a", "b"]), "Expected valid Queue"
    assert check_queue("", ["a", "b"]) == False, "Expected exception queue name is empty"
    assert check_queue(" ", ["a", "b"]) == False, "Expected Exception cannot have spaces for a name"
    assert check_queue(".", ["a", "b"]) == False, "Expected Exception cannot start name with a ."
    assert check_queue("m", []) == False, "Expected Exception queue items list is empty"

    assert check_event(1), "Expected valid Event"
    assert check_event(2), "Expected valid Event"
    assert check_event_number_and_name(2, "fred"), "Expected valid Event"
    assert check_event_number_and_name(2, 2) == False, "Expected failure since the name is not a string"

    assert check_repeat_date("m", 20000101, 20001201, 200), "Expected valid repeat"
    assert check_repeat_date("m", 20001201, 20000101, 200) == False, "Expected exception since end YMD > start YMD"
    assert check_repeat_date("m", 200001011, 20001201, 200) == False, "Expected Exception since start is invalid."
    assert check_repeat_date("m", 20000101, 200012013, 200) == False, "Expected Exception since send is invalid."
    assert check_repeat_date("m", 00000000, 00000000, 200) == False, "Expected Exception since start/end are not valid dates is invalid."
    assert check_repeat_date("", 20000101, 20001201, 200) == False, "Expected Exception since no name specified"
    assert check_repeat_date(" ", 20000101, 20001201, 200) == False, "Expected Exception cannot have spaces for a name"
    assert check_repeat_datetime("m", "20000101T000000", "20001201T000000", "4800:00:00"), "Expected valid repeat"
    assert check_repeat_datetime("m", "20001201T000000", "20000101T000000", "4800:00:00") == False, "Expected exception since end YMD > start YMD"
    assert check_repeat_datetime("m", "200001011T000000", "20001201T000000", "4800:00:00") == False, "Expected Exception since start is invalid."
    assert check_repeat_datetime("m", "20000101T000000", "200012013T000000", "4800:00:00") == False, "Expected Exception since send is invalid."
    assert check_repeat_datetime("m", "00000000T000000", "00000000T000000", "4800:00:00") == False, "Expected Exception since start/end are not valid dates is invalid."
    assert check_repeat_datetime("", "20000101T000000", "20001201T000000", "4800:00:00") == False, "Expected Exception since no name specified"
    assert check_repeat_datetime(" ", "20000101T000000", "20001201T000000", "4800:00:00") == False, "Expected Exception cannot have spaces for a name"
    assert check_repeat_integer("name", 0, 10, 2), "Expected valid repeat"
    assert check_repeat_integer("", 0, 10, 2) == False, "Expected Exception since no name specified"
    assert check_repeat_integer(" ", 0, 10, 2) == False, "Expected Exception cannot have spaces for a name"
    assert check_repeat_string("name", ["a"]), "Expected valid repeat"
    assert check_repeat_string("", ["a"]) == False, "Expected Exception since no name specified"
    assert check_repeat_string(" ", ["a"]) == False, "Expected Exception cannot have spaces for a name"
    assert check_repeat_string("name", [1, 2]) == False, "Expected Exception since a list of strings was expected"
    assert check_repeat_string("name", []) == False, "Expected Exception since list of strings is empty"
    assert check_repeat_enumerated("name", ["a"]), "Expected valid repeat"
    assert check_repeat_enumerated("", ["a"]) == False, "Expected Exception since no name specified"
    assert check_repeat_enumerated(" ", ["a"]) == False, "Expected Exception since no name specified"
    assert check_repeat_enumerated("name", [1, 2]) == False, "Expected Exception since a list of strings was expected"
    assert check_repeat_enumerated("name", []) == False, "Expected Exception since list is empty"

    assert check_variable("name", "value"), "Expected valid Variable"
    assert check_variable("name", ""), "Expected valid Variable"
    assert check_variable("name", " "), "Expected valid Variable"
    assert check_variable("name", "12"), "Expected valid Variable"
    assert check_variable("", "12") == False, "Expected Exception name must be specified"
    assert check_variable(" ", "12") == False, "Expected Exception cannot have spaces for a name"

    assert check_label("name", "value"), "Expected valid label"
    assert check_label("name", ""), "Expected valid label"
    assert check_label("name", " "), "Expected valid label"
    assert check_label("name", "12"), "Expected valid label"
    assert check_label("", "12") == False, "Expected exception name must be specified"
    assert check_label(" ", "12") == False, "Expected Exception cannot have spaces for a name"

    assert check_limit("name", 1), "Expected valid limit"
    assert check_limit("name", 20000), "Expected valid limit"
    assert check_limit("name", "ten") == False, "Expected exception, token must be a integer"
    assert check_limit("name", "2") == False, "Expected exception, token must be a integer"
    assert check_limit("", "2") == False, "Expected exception, no name specified"
    assert check_limit(" ", "2") == False, "Expected exception, cannot have spaces for a name"

    assert check_inlimit("limit_name", "/path/to/limit", 1), "Expected valid in limit"
    assert check_inlimit("limit_name", "/path/to/limit", 999999), "Expected valid in limit"
    assert check_inlimit("limit_name", "", 1), "Expected valid in limit"
    assert check_inlimit("", "", 1) == False, "Expected exception, no limit name specified"
    assert check_inlimit(" ", "", 1) == False, "Expected exception, cannot have spaces for a name"

    # ========================================================================
    print("Check node names")
    for i in range(25):
        assert check_node_name(str(i)), "Integer names should be allowed"

    for name in valid_names:
        assert check_node_name(name), "Expected valid name " + name

    for name in invalid_names:
        assert check_node_name(name) == False, "Expected exception for invalid name " + name

    assert check_defs("a_made_up_path_that_doesnt_not_exit.def") == False, "Expected exception, Defs file does not exist"

    # =================================================================================
    print("test save_as_defs")
    defs = Defs()  # create a empty definition
    s1 = defs.add_suite("s1")  # create a suite "s1" and add to defs
    defs.save_as_defs("testerror.def")  # create a defs on disk
    assert check_defs("testerror.def"), "Expected defs file to exist"
    os.remove("testerror.def")

    # =================================================================================
    print("Check duplicate suites not allowed")
    test_passed = False
    try:
        defs = Defs()  # create a empty definition
        s1 = defs.add_suite("s1")  # create a suite "s1" and add to defs
        s2 = defs.add_suite("s1")  # Exception thrown trying to add suite name "s1" again
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "duplicate suite test failed"

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_today("00:30")
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Adding today at the suite level should fail"
    print("check adding today at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_time("00:30")
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Adding time at the suite level should fail"
    print("check adding time at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_date(1, 1, 2016)
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Adding date at the suite level should fail"
    print("check adding date at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_day("monday")
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Adding day at the suite level should fail"
    print("check adding day at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_trigger("1 == 0")
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Adding a trigger at suite level should fail"
    print("check adding trigger at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_part_trigger("1 == 0")
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Adding a part trigger at suite level should fail"
    print("check adding part trigger at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_complete("1 == 0")
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Adding a complete trigger at suite level should fail"
    print("check adding complete trigger at the suite level: RuntimeError: ")

    # =================================================================================
    test_passed = False
    try:
        defs = Defs()
        defs.add_suite("1").add_part_complete("1 == 0")
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Adding a part complete trigger at suite level should fail"
    print("check adding part complete trigger at the suite level: RuntimeError: ")

    # =================================================================================
    print("check adding trigger referencing parent should fail ECFLOW-1436")
    defs = Defs()
    defs += Suite("obs") + Family("anon", Task("t1", Trigger("anon == complete")))
    assert len(defs.check()) > 0, "Adding a trigger referencing a parent without .. should fail"

    # =================================================================================
    print("check duplicate family not allowed")
    test_passed = False
    try:
        suite = Suite("1")
        fam1 = Family("1")
        fam2 = Family("1")
        suite.add_family(fam1)
        suite.add_family(fam2)
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "duplicate Family test failed"

    # =================================================================================
    print("duplicate task not allowed")
    test_passed = False
    try:
        suite = Suite("1")
        ta = Task("a")
        tb = Task("a")
        suite.add_task(ta)
        suite.add_task(tb)
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "duplicate Task test failed"

    # =================================================================================
    print("Task and family of same name should not be allowed")
    test_passed = False
    try:
        suite = Suite("1")
        ta = Task("a")
        tb = Family("a")
        suite.add_task(ta)
        suite.add_family(tb)
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Task and family of same name should not be allowed"

    # =================================================================================
    print("check duplicate meter not allowed")
    test_passed = False
    try:
        defs = Defs()
        suite = defs.add_suite("1")
        ta = suite.add_task("a")
        ta.add_meter("meter", 0, 100);
        ta.add_meter("meter", 0, 100);
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "duplicate meter test failed"

    # =================================================================================
    print("check duplicate queue not allowed")
    test_passed = False
    try:
        defs = Defs()
        suite = defs.add_suite("1")
        suite.add_queue("q", ["a"]);
        suite.add_queue("q", ["a", "b"]);
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "duplicate queue test failed"

    # =================================================================================
    print("check duplicate event not allowed")
    test_passed = False
    try:
        defs = Defs()
        suite = defs.add_suite("1")
        ta = suite.add_task("a")
        ta.add_event(1);
        ta.add_event("1");
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "duplicate event test failed"

    test_passed = False
    try:
        defs = Defs()
        suite = defs.add_suite("1")
        ta = suite.add_task("a")
        ta.add_event("name");
        ta.add_event("name");
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "duplicate event test failed"

    test_passed = False
    try:
        defs = Defs()
        suite = defs.add_suite("1")
        ta = suite.add_task("a")
        ta.add_event(1, "name");
        ta.add_event(1, "name");
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "duplicate event test failed"

    # =================================================================================
    print("check cannot add same node to different containers")
    defs1 = Defs();
    suite = defs1.add_suite("s1")
    family = suite.add_family("f1")
    task = family.add_task("t1")

    test_passed = False
    try:
        defs2 = Defs();
        defs2.add_suite(suite)
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Can't add same suite to two different defs"

    test_passed = False
    try:
        defs3 = Defs();
        suite = defs3.add_suite(suite)
        suite.add_family(family)
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Can't add same family to two different suites"

    test_passed = False
    try:
        defs4 = Defs();
        suite = defs4.add_suite("s1")
        suite.add_task(task)
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Can't add same task to two different containers"

    test_passed = False
    try:
        defs4 = Defs();
        suite = defs4.add_suite("s1")
        suite.add_autocancel(3)
        suite.add_autocancel(4)
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Can't add two autocancel on same node"

    test_passed = False
    try:
        defs4 = Defs();
        suite = defs4.add_suite("s1")
        suite.add_autoarchive(3)
        suite.add_autocancel(4)
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Can't add autocancel and autoarchive on same node"

    test_passed = False
    try:
        defs4 = Defs();
        suite = defs4.add_suite("s1")
        suite.add_autorestore(["/s1"])
        suite.add_autorestore(["/s1"])
    except RuntimeError as e:
        test_passed = True
        pass
    assert test_passed, "Can't add two autorestore on same node"
    print("All Tests pass")



#!/usr/bin/env python3

import pathlib
from ecflow import Defs, Suite, Family, Task, Edit

def create_family_f1():
    return Family("f1", Task("t1"), Task("t2"))

if __name__ == '__main__':

    base = pathlib.Path.home() / "course"

    print("[1] Creating suite definition")

    defs = Defs(
        Suite("test",
              Edit(ECF_INCLUDE=str(base), ECF_HOME=str(base)),
              create_family_f1()))
    print(defs)

    print("[2] Checking job creation: .ecf -> .job0")
    print(defs.check_job_creation())

    print("[3] Saving definition to file 'test.def'")
    defs.save_as_defs(str(base / "test.def"))



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

# This test ensures that creation of Defs files works as expected.

import os
import ecflow

from ecflow import Ecf, Client, debug_build

if __name__ == "__main__":
    print("####################################################################")
    print("Running TestEcf, ecflow version " + Client().version() + " debug build(" + str(debug_build()) + ")")
    print("####################################################################")

    default_debug_level = Ecf.debug_level()
    assert default_debug_level == 0, "Expected default debug level to be 0"
    Ecf.set_debug_level(10)
    assert Ecf.debug_level() == 10, "Expected debug level to be 10"
    Ecf.set_debug_level(0)
    assert Ecf.debug_level() == 0, "Expected debug level to be 0"

    assert Ecf.debug_equality() == False, "Expected  default for Ecf.debug_equality() == False "
    Ecf.set_debug_equality(True)
    assert Ecf.debug_equality() == True, "Expected  debug_equality() == True "
    Ecf.set_debug_equality(False)
    assert Ecf.debug_equality() == False, "Expected  Ecf.debug_equality() == False "

    print("All tests pass")



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

from ecflow import Alias, AttrType, Autocancel, CheckPt, ChildCmdType, Client, Clock, Cron, DState, Date, Day, Days, \
    Defs, Ecf, Event, Expression, Family, FamilyVec, File, Flag, FlagType, FlagTypeVec, InLimit, \
    JobCreationCtrl, Label, Late, Limit, Meter, Node, NodeContainer, NodeVec, PartExpression, PrintStyle, \
    Repeat, RepeatDate, RepeatDay, RepeatEnumerated, RepeatInteger, RepeatString, SState, State, Style, \
    Submittable, Suite, SuiteVec, Task, TaskVec, Time, TimeSeries, TimeSlot, Today, UrlCmd, Variable, \
    VariableList, Verify, WhyCmd, ZombieAttr, ZombieType, ZombieUserActionType, Trigger, Complete, Edit, Defstatus
import unittest
import sys
import os


class Test_default_defs(unittest.TestCase):

    def setUp(self):
        self.file_name = 'Test_default_defs.defs'
        defs = Defs() + Suite('s1')
        defs.save_as_defs(self.file_name)

    def test_defs_default_constructor(self):
        defs = Defs()
        # PrintStyle.set_style( Style.MIGRATE )
        # print defs
        self.assertEqual(defs.get_state(), State.unknown, "expected unknow state but found " + str(defs.get_state()))
        self.assertEqual(defs.get_server_state(), SState.RUNNING, "expected default server state tobe running state but found " + str(defs.get_server_state()))
        self.assertEqual(len(list(defs.suites)), 0, "expected no suites but found " + str(len(list(defs.suites))))

        defs = Defs(self.file_name)
        self.assertEqual(defs.get_state(), State.unknown, "expected unknow state but found " + str(defs.get_state()))
        self.assertEqual(defs.get_server_state(), SState.RUNNING, "expected default server state tobe running state but found " + str(defs.get_server_state()))
        self.assertEqual(len(list(defs.suites)), 1, "expected 1 suites but found " + str(len(list(defs.suites))))

    def tearDown(self):
        try:
            os.remove(self.file_name)
        except:
            pass


class Test_defs_raw_constructor(unittest.TestCase):

    def test_defs_raw_constructor_errors(self):
        with self.assertRaises(RuntimeError):
            defs = Defs("file_name.def", Suite('s1'))
        with self.assertRaises(RuntimeError):
            defs = Defs("file_name.def", Edit(a='b'))
        with self.assertRaises(RuntimeError):
            defs = Defs("file_name.def", {'a': 'b'})
        with self.assertRaises(RuntimeError):
            defs = Defs("file_name.def", Variable('a', 'b'))
        with self.assertRaises(RuntimeError):
            defs = Defs("file_name.def", [Variable('a', 'b')])

    def test_defs_raw_constructor_suite_creation(self):
        defs = Defs(Suite('s1'))
        self.assertEqual(len(list(defs.suites)), 1, "expected 1 suites but found " + str(len(list(defs.suites))))

        defs = Defs(Suite('s1'), Suite('s2'), Suite('s3'))
        self.assertEqual(len(list(defs.suites)), 3, "expected 3 suites but found " + str(len(list(defs.suites))))

        defs = Defs([Suite('s{0}'.format(i)) for i in range(1, 5)])
        self.assertEqual(len(list(defs.suites)), 4, "expected 4 suites but found " + str(len(list(defs.suites))))

    def test_defs_raw_constructor_variables(self):
        defs = Defs(Edit(a='b'))
        self.assertEqual(len(list(defs.user_variables)), 1, "expected 1 user variable but found " + str(len(list(defs.user_variables))))

        defs = Defs(Variable('a', 'b'))
        self.assertEqual(len(list(defs.user_variables)), 1, "expected 1 user variable but found " + str(len(list(defs.user_variables))))

        defs = Defs({'a': 'a', 'b': 'b'})
        self.assertEqual(len(list(defs.user_variables)), 2, "expected 2 user variable but found " + str(len(list(defs.user_variables))))

        defs = Defs(a='a', b='b')  # uses kwy word arguments
        self.assertEqual(len(list(defs.user_variables)), 2, "expected 2 user variable but found " + str(len(list(defs.user_variables))))

        defs = Defs(Edit(a='b'), Variable('b', 'b'), {'c': 'a', 'd': 'b'}, e='a', f='b')
        self.assertEqual(len(list(defs.user_variables)), 6, "expected 6 user variable but found " + str(len(list(defs.user_variables))))

    def test_defs_raw_constructor(self):
        defs = Defs(Suite('s1'), Edit(a='b'))
        self.assertEqual(len(list(defs.user_variables)), 1, "expected 1 user variable but found " + str(len(list(defs.user_variables))))
        self.assertEqual(len(list(defs.suites)), 1, "expected 1 suites but found " + str(len(list(defs.suites))))

        defs = Defs(Suite('s1'), Suite('s2'), Suite('s3'), Variable('a', 'b'))
        self.assertEqual(len(list(defs.user_variables)), 1, "expected 1 user variable but found " + str(len(list(defs.user_variables))))
        self.assertEqual(len(list(defs.suites)), 3, "expected 3 suites but found " + str(len(list(defs.suites))))

        defs = Defs([Suite('s{0}'.format(i)) for i in range(1, 5)], {'a': 'a', 'b': 'b'})
        self.assertEqual(len(list(defs.user_variables)), 2, "expected 2 user variable but found " + str(len(list(defs.user_variables))))
        self.assertEqual(len(list(defs.suites)), 4, "expected 4 suites but found " + str(len(list(defs.suites))))

        defs = Defs(Suite('s1'), Suite('s2'), Suite('s3'), a='a', b='b')  # uses kwy word arguments
        self.assertEqual(len(list(defs.user_variables)), 2, "expected 2 user variable but found " + str(len(list(defs.user_variables))))
        self.assertEqual(len(list(defs.suites)), 3, "expected 3 suites but found " + str(len(list(defs.suites))))

        defs = Defs(Suite('s1'), Edit(a='b'), Variable('b', 'b'), {'c': 'a', 'd': 'b'}, e='a', f='b')
        self.assertEqual(len(list(defs.user_variables)), 6, "expected 6 user variable but found " + str(len(list(defs.user_variables))))
        self.assertEqual(len(list(defs.suites)), 1, "expected 1 suites but found " + str(len(list(defs.suites))))

    def test_defs_constructor(self):
        defs = Defs(Suite('s1'), Edit(a='b'))
        defs2 = Defs() + Suite('s1') + Edit(a='b')
        defs3 = Defs()
        defs3 += Suite('s1')
        defs3 += Edit(a='b')
        self.assertEqual(defs, defs2, "defs not equal\n" + str(defs) + "\n\n" + str(defs2))
        self.assertEqual(defs, defs3, "defs not equal\n" + str(defs) + "\n\n" + str(defs3))

        defs = Defs(Suite('s1'), Suite('s2'), Suite('s3'), Variable('a', 'b'))
        defs2 = Defs() + Suite('s1') + Suite('s2') + Suite('s3') + Variable('a', 'b')
        defs3 = Defs().add(Suite('s1'), Suite('s2'), Suite('s3'), Variable('a', 'b'))
        self.assertEqual(defs, defs2, "defs not equal\n" + str(defs) + "\n\n" + str(defs2))
        self.assertEqual(defs, defs3, "defs not equal\n" + str(defs) + "\n\n" + str(defs3))

        defs = Defs([Suite('s{0}'.format(i)) for i in range(1, 5)], {'a': 'a'})
        defs2 = Defs() + [Suite('s{0}'.format(i)) for i in range(1, 5)] + {'a': 'a'}
        defs3 = Defs()
        for i in range(1, 5):
            defs3.add_suite('s{0}'.format(i))
        defs3.add_variable('a', 'a')
        self.assertEqual(defs, defs2, "defs not equal\n" + str(defs) + "\n\n" + str(defs2))
        self.assertEqual(defs, defs3, "defs not equal\n" + str(defs) + "\n\n" + str(defs3))

        defs = Defs(Suite('s1'), Edit(a='a'), Variable('b', 'b'), {'c': 'c'}, {'d': 'd'})
        defs2 = Defs().add(Suite('s1'), Edit(a='a'), Variable('b', 'b'), {'c': 'c'}, {'d': 'd'})
        defs3 = Defs()
        defs3.add_suite('s1')
        defs3.add_variable('a', 'a')
        defs3.add_variable('b', 'b')
        defs3.add_variable('c', 'c')
        defs3.add_variable('d', 'd')
        self.assertEqual(defs, defs2, "defs not equal\n" + str(defs) + "\n\n" + str(defs2))
        self.assertEqual(defs, defs3, "defs not equal\n" + str(defs) + "\n\n" + str(defs3))


if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

# This test ensures that the nodes are printed using the expected format.

from ecflow import Defs, Suite, Task, Family, Client
import ecflow_test_util as Test
import os


def test_print_defs():
    defs = Defs()
    with Suite("s") as s:
        defs.add_suite(s)
        with Family("f") as f:
            s.add_family(f)
            with Task("t") as t:
                f.add_task(t)
                t.add_variable("NAME", "VALUE")

                expected = f"#{Client().version()}\nsuite s\n  family f\n    task t\n      edit NAME 'VALUE'\n  endfamily\nendsuite\n# enddef\n"
                actual = defs.__str__()
                assert actual == expected, "Expected:\n" + expected + "\n" + "Actual:\n" + actual


def test_print_suite():
    with Suite("s") as s:
        with Family("f") as f:
            s.add_family(f)
            with Task("t") as t:
                f.add_task(t)
                t.add_variable("NAME", "VALUE")

                expected = "suite s\n  family f\n    task t\n      edit NAME 'VALUE'\n  endfamily\nendsuite\n"
                actual = s.__str__()
                assert actual == expected, "Expected:\n" + expected + "\n" + "Actual:\n" + actual


def test_print_family():
    with Family("f") as f:
        with Task("t") as t:
            f.add_task(t)
            t.add_variable("NAME", "VALUE")

            expected = "  family f\n    task t\n      edit NAME 'VALUE'\n  endfamily\n"
            actual = f.__str__()
            assert actual == expected, "Expected:\n" + expected + "\n" + "Actual:\n" + actual


def test_print_task():
    with Task("t") as t:
        t.add_variable("NAME", "VALUE")

        expected = "  task t\n    edit NAME 'VALUE'\n"
        actual = t.__str__()
        assert actual == expected, "Expected:\n" + expected + "\n" + "Actual:\n" + actual


if __name__ == "__main__":
    Test.print_test_start(os.path.basename(__file__))

    test_print_defs()
    test_print_suite()
    test_print_family()
    test_print_task()

    print("All tests pass")



#
# Copyright 2009- ECMWF.
#
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#

# This test ensures the SSL setup for the ecFlow client works as expected.

import os

import ecflow_test_util as Test
import inspect

import ecflow as ecf

port = 31415
host = os.uname()[1]


class Tests:

    @staticmethod
    def can_setup_client_without_ssl_and_get_empty_certificate():
        Test.name_this_test()

        expected_crt = ""

        client = ecf.Client(host, port)
        crt = client.get_certificate()
        assert crt is expected_crt, f"Expected certificate to be '{expected_crt}', but got '{crt}'"


    @staticmethod
    def can_setup_client_with_ssl_with_shared_certificate_solely_based_envvars_and_ecf_ssl_empty():
        Test.name_this_test()

        expected_crt = f"server.crt"

        with Test.MockFile(expected_crt) as crt_file:
            with Test.MockEnvironment({"ECF_SSL": "", "ECF_HOST": host, "ECF_PORT": str(port)}) as env:
                client = ecf.Client()
                crt = client.get_certificate()
                assert crt.endswith(expected_crt), f"Expected certificate to end with '{expected_crt}', but got {crt}"

    @staticmethod
    def can_setup_client_with_ssl_with_shared_certificate_solely_based_envvars():
        Test.name_this_test()

        expected_crt = f"server.crt"

        with Test.MockFile(expected_crt) as crt_file:
            with Test.MockEnvironment({"ECF_SSL": "1", "ECF_HOST": host, "ECF_PORT": str(port)}) as env:
                client = ecf.Client()
                crt = client.get_certificate()
                assert crt.endswith(expected_crt), f"Expected certificate to end with '{expected_crt}', but got {crt}"

    @staticmethod
    def can_setup_client_with_ssl_requesting_shared_but_finding_specific_certificate_solely_based_envvars():
        Test.name_this_test()

        expected_crt = f"{host}.{port}.crt"

        with Test.MockFile(expected_crt) as crt_file:
            with Test.MockEnvironment({"ECF_SSL": "1", "ECF_HOST": host, "ECF_PORT": str(port)}) as env:
                client = ecf.Client()
                crt = client.get_certificate()
                assert crt.endswith(expected_crt), f"Expected certificate to end with '{expected_crt}', but got {crt}"

    @staticmethod
    def can_setup_client_with_ssl_with_specific_certificate_solely_based_envvars():
        Test.name_this_test()

        expected_crt = f"{host}.{port}.crt"

        with Test.MockFile(expected_crt) as crt_file:
            with Test.MockEnvironment({"ECF_SSL": "X", "ECF_HOST": host, "ECF_PORT": str(port)}) as env:
                client = ecf.Client()
                crt = client.get_certificate()
                assert crt.endswith(expected_crt), f"Expected certificate to end with '{expected_crt}', but got {crt}"

    @staticmethod
    def can_setup_client_with_ssl_with_shared_certificate_using_envvar():
        Test.name_this_test()

        expected_crt = f"server.crt"

        with Test.MockFile(expected_crt) as crt_file:
            with Test.MockEnvironment({"ECF_SSL": "1"}) as env:
                client = ecf.Client(host, port)
                crt = client.get_certificate()
                assert crt.endswith(expected_crt), f"Expected certificate to end with '{expected_crt}', but got {crt}"

    @staticmethod
    def can_setup_client_with_ssl_with_shared_certificate_using_empty_envvar():
        Test.name_this_test()

        expected_crt = f"server.crt"

        with Test.MockFile(expected_crt) as crt_file:
            with Test.MockEnvironment({"ECF_SSL": ""}) as env:
                client = ecf.Client(host, port)
                crt = client.get_certificate()
                assert crt.endswith(expected_crt), f"Expected certificate to end with '{expected_crt}', but got {crt}"

    @staticmethod
    def can_setup_client_with_ssl_with_shared_certificate_using_explicit_option():
        Test.name_this_test()

        expected_crt = f"server.crt"

        with Test.MockFile(expected_crt) as crt_file:
            client = ecf.Client(host, port)
            client.enable_ssl()
            crt = client.get_certificate()
            assert crt.endswith(expected_crt), f"Expected certificate to end with '{expected_crt}', but got {crt}"

    @staticmethod
    def can_setup_client_with_ssl_with_specific_certificate_using_envvar():
        Test.name_this_test()

        expected_crt = f"{host}.{port}.crt"

        with Test.MockFile(expected_crt) as crt_file:
            with Test.MockEnvironment({"ECF_SSL": "X"}) as env:
                client = ecf.Client(host, port)
                crt = client.get_certificate()
                assert crt.endswith(expected_crt), f"Expected certificate to end with '{expected_crt}', but got {crt}"

    @staticmethod
    def can_setup_client_with_ssl_with_specific_certificate_using_envvar_and_explicit_option():
        Test.name_this_test()

        expected_crt = f"{host}.{port}.crt"

        with Test.MockFile(expected_crt) as crt_file:
            with Test.MockEnvironment({"ECF_SSL": "X"}) as env:
                client = ecf.Client(host, port)
                client.enable_ssl()
                crt = client.get_certificate()
                assert crt.endswith(expected_crt), f"Expected certificate to end with '{expected_crt}', but got {crt}"

    @staticmethod
    def can_setup_client_with_ssl_requesting_shared_certificate_using_envvar_but_falling_back_to_specific_certificate ():
        Test.name_this_test()

        expected_crt = f"{host}.{port}.crt"

        with Test.MockFile(expected_crt) as crt_file:
            with Test.MockEnvironment({"ECF_SSL": "1"}) as env:
                client = ecf.Client(host, port)
                crt = client.get_certificate()
                assert crt.endswith(expected_crt), f"Expected certificate to end with '{expected_crt}', but got {crt}"


if __name__ == "__main__":
    Test.execute_all(Tests)
